function [parent]=branch(n_gen, prob_vec)
% BRANCH Simulate a branching process and plot it as a tree.
%   Each individual can get 0-N children with probabilities 
%   p0-pN.
%
% [parent] = branch(n_gen, prob_vec)
%
% Inputs: 
%   n_gen - number of generations to proceed
%   prob_vec - vector of probabilities for children. Note
%     that the first entry is a probability for 0.
% 
% Outputs: 
%   parent - a random tree in a random order. parent(i) is the index
%     number of the parent to the node i. 
%

% Authors: R.Gaigalas, I.Kaj
% v1.4 07-Nov-01

 if (nargin<1) % default parameter values
   n_gen = 6;
   prob_vec = [0.2 0.3 0.1 0.4];
 end
 
 max_kids = length(prob_vec)-1 % maximal number of children
 cum_prob = [cumsum(prob_vec) 1]; % distribution function
 parent = 0; 
 extinct = 0;
 n_kids = 1; % number of children in the last generation
 
 for i=1:n_gen
   
   % indexes of the parents
   par_ind = length(parent)-n_kids+1:length(parent);
   n_parents = n_kids;

   % generate a set of numbers of children for all individuals
   % according to the distribution
   coin = rand(1, n_parents); 
   
   % add the kids to the tree   
   n_kids = 0;
   for j=1:max_kids
     % find indexes of the parents who give birth to j children 
     j_ind = par_ind(find((coin>cum_prob(j)) ...
                     & (coin<=cum_prob(j+1))));
     if (~isempty(j_ind))
       parent = [parent repmat(j_ind, 1, j)];
       n_kids = n_kids+length(j_ind)*j;
     end
   end

   % exit if no individuals left 
   if (n_kids==0)
     extinct = 1;
     break;
   end   
 end
 
 if (extinct)
   fprintf('Extinct in generation %d\n', i);
 else
   fprintf('Stopped in generation %d\n', n_gen);
 end
 
 figure(1);
 trimtreeplot(parent);   
% figure(2);
% newtreeplot(parent);    
 
