function [adj_matr, n_coord]=waxtop(lambda, alpha, beta, domain) 
% WAXTOP Simulate and plot a random network topology by the method
%   suggested by Waxman (1988):
%   - nodes are a Poisson process in the plane with scaled
%    Lebesgue mean measure 
%   - nodes u and v are connected with probability
%    P(u,v)=alpha*exp(-d(u, v)/(beta*L)),
%    where alpha>0, beta<=1, d(u,v) is Euclidean distance, 
%    L is the maximum distance between any two nodes
%
% [adj_matr, n_coord] = waxtop(lambda,alpha, beta, domain) 
%
% Inputs: 
%   lambda - intensity of the Poisson process
%   alpha - maximal link probability
%   beta - parameter to control length of the edges. Increased <beta>
%     yields a larger ratio of long edges to short edges
%   domain - bounds for the region. A 4-dimensional vector in
%     the form [x_min x_max y_min y_max]. 
%
% Outputs:
%   adj_matr - adjacency matrix of the graph of the topology
%   n_coord - coordinates of the nodes

% Authors: R.Gaigalas, I.Kaj
% v1.4 07-Nov-01

 if (nargin<4) % default parameter values
   lambda = 0.6; % intensity of the Poisson process
   alpha = 0.4; % parameter for the link probability
   beta = 0.1; % parameter for the link probability
   domain = [0 10 0 10]; % bounds for the "geografical" domain
 end

 x_min = domain(1); 
 x_max = domain(2);
 y_min = domain(3);
 y_max = domain(4);
 clear domain;
   
 % number of points is Poisson distributed 
 % with intensity proportional to the area
 area = (x_max-x_min)*(y_max-y_min); 
 n_points = poissrnd(lambda*area)
% n_points = 20;
 
 %given the number of points, nodes are uniformly distributed
 n_coord = rand(n_points, 2);
 n_coord(:,1) = n_coord(:,1)*(x_max-x_min)+x_min;
 n_coord(:,2) = n_coord(:,2)*(y_max-y_min)+y_min;
 
 % create a matrix with distances
% x_rep = n_coord(:, 1)*ones(1, n_points);
% y_rep = n_coord(:, 2)*ones(1, n_points);
 x_rep = repmat(n_coord(:, 1), 1, n_points);
 y_rep = repmat(n_coord(:, 2), 1, n_points);
 dist_matr = sparse(triu(((x_rep-x_rep').^2+(y_rep-y_rep').^2).^0.5, 1));
 
 % create the matrix of probabilities
 prob_matr = alpha*spfun('exp', -dist_matr./(beta*max(max(dist_matr))));

 % generate the adjacency matrix
 coin = sprand(dist_matr);
 adj_matr = (coin>0) & (coin < prob_matr);

 % test for connectivity
% s_matr = speye(size(adj_matr));
% for i=1:n_points-1
%   s_matr = s_matr+adj_matr^i;
% end
% length(find(s_matr==0))
 
 % plot the network
 plot(n_coord(:,1), n_coord(:,2), '.');
 
 hold on;
 gplot(adj_matr, n_coord);
 hold off;
